package be.iminds.ilabt.jfed.highlevel.util;

import org.slf4j.Logger;
import org.slf4j.Marker;
import org.slf4j.helpers.MessageFormatter;

import javax.annotation.Nonnull;

public class LogEntryGeneratorWrappingLogger implements org.slf4j.Logger {
    private final org.slf4j.Logger wrappedLog;
    private final LogEntryListener logEntryListener;

    public LogEntryGeneratorWrappingLogger(@Nonnull Logger wrappedLog,
                                           @Nonnull LogEntryListener logEntryListener) {
        this.wrappedLog = wrappedLog;
        this.logEntryListener = logEntryListener;
    }

    @Override
    public String getName() {
        return wrappedLog.getName();
    }

    @Override
    public boolean isTraceEnabled() {
        return wrappedLog.isTraceEnabled();
    }

    @Override
    public void trace(String s) {
        wrappedLog.trace(s);
    }

    @Override
    public void trace(String s, Object o) {
        wrappedLog.trace(s, o);
    }

    @Override
    public void trace(String s, Object o, Object o1) {
        wrappedLog.trace(s, o, o1);
    }

    @Override
    public void trace(String s, Object... objects) {
        wrappedLog.trace(s, objects);
    }

    @Override
    public void trace(String s, Throwable throwable) {
        wrappedLog.trace(s, throwable);
    }

    @Override
    public boolean isTraceEnabled(Marker marker) {
        return wrappedLog.isTraceEnabled(marker);
    }

    @Override
    public void trace(Marker marker, String s) {
        wrappedLog.trace(marker, s);
    }

    @Override
    public void trace(Marker marker, String s, Object o) {
        wrappedLog.trace(marker, s, o);
    }

    @Override
    public void trace(Marker marker, String s, Object o, Object o1) {
        wrappedLog.trace(marker, s, o, o1);
    }

    @Override
    public void trace(Marker marker, String s, Object... objects) {
        wrappedLog.trace(marker, s, objects);
    }

    @Override
    public void trace(Marker marker, String s, Throwable throwable) {
        wrappedLog.trace(marker, s, throwable);
    }

    @Override
    public boolean isDebugEnabled() {
        return wrappedLog.isDebugEnabled();
    }

    @Override
    public void debug(String s) {
        wrappedLog.debug(s);
        logEntryListener.debug(s);
    }

    private static String makeMsg(String s, Object o) {
        return MessageFormatter.format(s, o).getMessage();
    }
    private static String makeMsg(String s, Object o1, Object o2) {
        return MessageFormatter.format(s, o1, o2).getMessage();
    }
    private static String makeArrMsg(String s, Object... o) {
        return MessageFormatter.arrayFormat(s, o).getMessage();
    }

    @Override
    public void debug(String s, Object o) {
        wrappedLog.debug(s, o);
        logEntryListener.debug(makeMsg(s, o));
    }

    @Override
    public void debug(String s, Object o, Object o1) {
        wrappedLog.debug(s, o, o1);
        logEntryListener.debug(makeMsg(s, o, o1));
    }

    @Override
    public void debug(String s, Object... objects) {
        wrappedLog.debug(s, objects);
        logEntryListener.debug(makeArrMsg(s, objects));
    }

    @Override
    public void debug(String s, Throwable throwable) {
        wrappedLog.debug(s, throwable);
        logEntryListener.debug(s, throwable);
    }

    @Override
    public boolean isDebugEnabled(Marker marker) {
        return wrappedLog.isDebugEnabled(marker);
    }

    @Override
    public void debug(Marker marker, String s) {
        wrappedLog.debug(marker, s);
        logEntryListener.debug(s);
    }

    @Override
    public void debug(Marker marker, String s, Object o) {
        wrappedLog.debug(marker, s, o);
        logEntryListener.debug(makeMsg(s, o));
    }

    @Override
    public void debug(Marker marker, String s, Object o, Object o1) {
        wrappedLog.debug(marker, s, o, o1);
        logEntryListener.debug(makeMsg(s, o, o1));
    }

    @Override
    public void debug(Marker marker, String s, Object... objects) {
        wrappedLog.debug(marker, s, objects);
        logEntryListener.debug(makeArrMsg(s, objects));
    }

    @Override
    public void debug(Marker marker, String s, Throwable throwable) {
        wrappedLog.debug(marker, s, throwable);
        logEntryListener.debug(s, throwable);
    }

    @Override
    public boolean isInfoEnabled() {
        return wrappedLog.isInfoEnabled();
    }

    @Override
    public void info(String s) {
        wrappedLog.info(s);
        logEntryListener.info(s);
    }

    @Override
    public void info(String s, Object o) {
        wrappedLog.info(s, o);
        logEntryListener.info(makeMsg(s, o));
    }

    @Override
    public void info(String s, Object o, Object o1) {
        wrappedLog.info(s, o, o1);
        logEntryListener.info(makeMsg(s, o, o1));
    }

    @Override
    public void info(String s, Object... objects) {
        wrappedLog.info(s, objects);
        logEntryListener.info(makeArrMsg(s, objects));
    }

    @Override
    public void info(String s, Throwable throwable) {
        wrappedLog.info(s, throwable);
        logEntryListener.info(s, throwable);
    }

    @Override
    public boolean isInfoEnabled(Marker marker) {
        return wrappedLog.isInfoEnabled(marker);
    }

    @Override
    public void info(Marker marker, String s) {
        wrappedLog.info(marker, s);
        logEntryListener.info(s);
    }

    @Override
    public void info(Marker marker, String s, Object o) {
        wrappedLog.info(marker, s, o);
        logEntryListener.info(makeMsg(s, o));
    }

    @Override
    public void info(Marker marker, String s, Object o, Object o1) {
        wrappedLog.info(marker, s, o, o1);
        logEntryListener.info(makeMsg(s, o, o1));
    }

    @Override
    public void info(Marker marker, String s, Object... objects) {
        wrappedLog.info(marker, s, objects);
        logEntryListener.info(makeArrMsg(s, objects));
    }

    @Override
    public void info(Marker marker, String s, Throwable throwable) {
        wrappedLog.info(marker, s, throwable);
        logEntryListener.info(s, throwable);
    }

    @Override
    public boolean isWarnEnabled() {
        return wrappedLog.isWarnEnabled();
    }

    @Override
    public void warn(String s) {
        wrappedLog.warn(s);
        logEntryListener.warn(s);
    }

    @Override
    public void warn(String s, Object o) {
        wrappedLog.warn(s, o);
        logEntryListener.warn(makeMsg(s, o));
    }

    @Override
    public void warn(String s, Object... objects) {
        wrappedLog.warn(s, objects);
        logEntryListener.warn(makeArrMsg(s, objects));
    }

    @Override
    public void warn(String s, Object o, Object o1) {
        wrappedLog.warn(s, o, o1);
        logEntryListener.warn(makeMsg(s, o, o1));
    }

    @Override
    public void warn(String s, Throwable throwable) {
        wrappedLog.warn(s, throwable);
        logEntryListener.warn(s, throwable);
    }

    @Override
    public boolean isWarnEnabled(Marker marker) {
        return wrappedLog.isWarnEnabled(marker);
    }

    @Override
    public void warn(Marker marker, String s) {
        wrappedLog.warn(marker, s);
        logEntryListener.warn(s);
    }

    @Override
    public void warn(Marker marker, String s, Object o) {
        wrappedLog.warn(marker, s, o);
        logEntryListener.warn(makeMsg(s, o));
    }

    @Override
    public void warn(Marker marker, String s, Object o, Object o1) {
        wrappedLog.warn(marker, s, o, o1);
        logEntryListener.warn(makeMsg(s, o, o1));
    }

    @Override
    public void warn(Marker marker, String s, Object... objects) {
        wrappedLog.warn(marker, s, objects);
        logEntryListener.warn(makeArrMsg(s, objects));
    }

    @Override
    public void warn(Marker marker, String s, Throwable throwable) {
        wrappedLog.warn(marker, s, throwable);
        logEntryListener.warn(s, throwable);
    }

    @Override
    public boolean isErrorEnabled() {
        return wrappedLog.isErrorEnabled();
    }

    @Override
    public void error(String s) {
        wrappedLog.error(s);
        logEntryListener.error(s);
    }

    @Override
    public void error(String s, Object o) {
        wrappedLog.error(s, o);
        logEntryListener.error(makeMsg(s, o));
    }

    @Override
    public void error(String s, Object o, Object o1) {
        wrappedLog.error(s, o, o1);
        logEntryListener.error(makeMsg(s, o, o1));
    }

    @Override
    public void error(String s, Object... objects) {
        wrappedLog.error(s, objects);
        logEntryListener.error(makeArrMsg(s, objects));
    }

    @Override
    public void error(String s, Throwable throwable) {
        wrappedLog.error(s, throwable);
        logEntryListener.error(s, throwable);
    }

    @Override
    public boolean isErrorEnabled(Marker marker) {
        return wrappedLog.isErrorEnabled(marker);
    }

    @Override
    public void error(Marker marker, String s) {
        wrappedLog.error(marker, s);
        logEntryListener.error(s);
    }

    @Override
    public void error(Marker marker, String s, Object o) {
        wrappedLog.error(marker, s, o);
        logEntryListener.error(makeMsg(s, o));
    }

    @Override
    public void error(Marker marker, String s, Object o, Object o1) {
        wrappedLog.error(marker, s, o, o1);
        logEntryListener.error(makeMsg(s, o, o1));
    }

    @Override
    public void error(Marker marker, String s, Object... objects) {
        wrappedLog.error(marker, s, objects);
        logEntryListener.error(makeArrMsg(s, objects));
    }

    @Override
    public void error(Marker marker, String s, Throwable throwable) {
        wrappedLog.error(marker, s, throwable);
        logEntryListener.error(s, throwable);
    }
}
